import { Virtualization } from './virtualization';
import { ModuleDeclaration } from '@syncfusion/ej2-base';
import { Component, BaseEventArgs } from '@syncfusion/ej2-base';
import { INotifyPropertyChanged, ChildProperty } from '@syncfusion/ej2-base';
import { EmitType } from '@syncfusion/ej2-base';
import { DataManager, Query } from '@syncfusion/ej2-data';
import { ListBaseOptions, SortOrder, FieldsMapping } from '../common/list-base';
import { ListViewModel, FieldSettingsModel } from './list-view-model';
export declare const classNames: ClassNames;
export interface Fields {
    /**
     * ID attribute of specific list-item.
     */
    id?: string | number;
    /**
     * It is used to map the text value of list item from the dataSource.
     */
    text?: string | number;
    /**
     * It is used to map the custom field values of list item from the dataSource.
     */
    [key: string]: Object | string | number | undefined;
}
export declare class FieldSettings extends ChildProperty<FieldSettings> {
    /**
     * ID attribute of specific list-item.
     */
    id: string;
    /**
     * It is used to map the text value of list item from the dataSource.
     */
    text: string;
    /**
     * This property used to check whether the list item is in checked state or not.
     */
    isChecked: string;
    /**
     * To check whether the visibility state of list item.
     */
    isVisible: string;
    /**
     * It is used to enable the list item
     */
    enabled: string;
    /**
     * It is used to customize the icon to the list items dynamically.
     *  We can add specific image to the icons using iconCss property.
     */
    iconCss: string;
    /**
     * This property used for nested navigation of list-items.
     * Refer the documentation [here](./listview/nested-list)
     *  to know more about this property with demo.
     */
    child: string;
    /**
     * It is used to display `tooltip content of text` while hovering on list items.
     */
    tooltip: string;
    /**
     * It wraps the list view element into a group based on the value of groupBy property.
     * Refer the documentation [here](./listview/grouping)
     *  to know more about this property with demo.
     */
    groupBy: string;
    /**
     * It is used to enable the sorting of list items to be ascending or descending.
     */
    sortBy: string;
    /**
     * Defines the HTML attributes such as id, class, etc,. for the specific list item.
     */
    htmlAttributes: string;
    /**
     * It is used to fetch a specified named table data while using serviceUrl of DataManager
     *  in dataSource property.
     * Refer the documentation [here](https://ej2.syncfusion.com/documentation/data/getting-started?lang=typescript)
     *  to know more about this property with demo.
     */
    tableName: string;
}
/**
 * Animation configuration settings.
 */
export interface AnimationSettings {
    /**
     * It is used to specify the effect which is shown in sub list transform.
     */
    effect?: ListViewEffect;
    /**
     * It is used to specify the time duration of transform object.
     */
    duration?: number;
    /**
     * It is used to specify the easing effect applied while transform
     */
    easing?: string;
}
/**
 * ListView animation effects
 */
export declare type ListViewEffect = 'None' | 'SlideLeft' | 'SlideDown' | 'Zoom' | 'Fade';
/**
 * ListView check box positions
 */
export declare type checkBoxPosition = 'Left' | 'Right';
/**
 * Represents the EJ2 ListView control.
 * ```html
 * <div id="listview">
 * <ul>
 * <li>Favourite</li>
 * <li>Documents</li>
 * <li>Downloads</li>
 * </ul>
 * </div>
 * ```
 * ```typescript
 *   var lvObj = new ListView({});
 *   lvObj.appendTo("#listview");
 * ```
 */
export declare class ListView extends Component<HTMLElement> implements INotifyPropertyChanged {
    private ulElement;
    private selectedLI;
    private onUIScrolled;
    private curUL;
    private curDSLevel;
    private curViewDS;
    private curDSJSON;
    localData: {
        [key: string]: Object;
    }[];
    private liCollection;
    private headerEle;
    private contentContainer;
    private touchModule;
    private listBaseOption;
    virtualizationModule: Virtualization;
    private animateOptions;
    private rippleFn;
    private isNestedList;
    private currentLiElements;
    private selectedData;
    private selectedId;
    private isWindow;
    private selectedItems;
    private aniObj;
    /**
     * This cssClass property helps to use custom skinning option for ListView component,
     *  by adding the mentioned class name into root element of ListView.
     * @default ''
     */
    cssClass: string;
    /**
     * It enables UI virtualization which will increase ListView performance on loading large number of data.
     *
     * @default false
     */
    enableVirtualization: boolean;
    /**
     * Defines the HTML attributes such as id, class, etc., for the ListView.
     * @default {}
     */
    htmlAttributes: {
        [key: string]: string;
    };
    /**
     * It specifies enabled state of ListView component.
     * @default true
     */
    enable: boolean;
    /**
     * It provides the data to render the ListView component which is mapped
     *  with the fields of ListView.
     *
     * {% codeBlock src="listview/datasource-api/index.ts" %}{% endcodeBlock %}
     * @default []
     */
    dataSource: {
        [key: string]: Object;
    }[] | string[] | number[] | DataManager;
    /**
     * It is used to fetch the specific data from dataSource by using where, select key words.
     * Refer the documentation [here]
     * (./data-binding#bind-to-remote-data)
     *  to know more about this property with demo.
     *
     * {% codeBlock src="listview/query-api/index.ts" %}{% endcodeBlock %}
     * @default null
     */
    query: Query;
    /**
     * It is used to map keys from the dataSource which extracts the appropriate data from the dataSource
     *  with specified mapped with the column fields to render the ListView.
     *
     * {% codeBlock src="listview/fields-api/index.ts" %}{% endcodeBlock %}
     * @default ListBase.defaultMappedFields
     */
    fields: FieldSettingsModel;
    /**
     * It is used to apply the animation to sub list navigation of list items.
     * @default { effect: 'SlideLeft', duration: 400, easing: 'ease' }
     */
    animation: AnimationSettings;
    /**
     * It is used to enable the sorting of list items to be ascending or descending.
     *
     * {% codeBlock src="listview/sortorder-api/index.ts" %}{% endcodeBlock %}
     * @default 'None'
     */
    sortOrder: SortOrder;
    /**
     * Using this property, we can show or hide the icon of list item.
     *
     * {% codeBlock src="listview/showicon-api/index.ts" %}{% endcodeBlock %}
     * @default false
     */
    showIcon: boolean;
    /**
     * Using this property, we can show or hide the `checkbox`.
     *
     * {% codeBlock src="listview/showcheckbox-api/index.ts" %}{% endcodeBlock %}
     * @default false
     */
    showCheckBox: boolean;
    /**
     * It is used to set the position of check box in an item.
     * @default 'Left'
     */
    checkBoxPosition: checkBoxPosition;
    /**
     * It is used to set the title of ListView component.
     *
     * {% codeBlock src="listview/fields-api/index.ts" %}{% endcodeBlock %}
     * @default ""
     */
    headerTitle: string;
    /**
     * Using this property, we can show or hide the header of ListView component.
     *
     * {% codeBlock src="listview/fields-api/index.ts" %}{% endcodeBlock %}
     * @default false
     */
    showHeader: boolean;
    /**
     * It is used to set the height of the ListView component.
     * @default ''
     */
    height: number | string;
    /**
     * It sets the width to the ListView component.
     * @default ''
     */
    width: number | string;
    /**
     * The ListView supports to customize the content of each list items with the help of template property.
     * Refer the documentation [here](./listview/customizing-templates)
     *  to know more about this property with demo.
     *
     * {% codeBlock src="listview/template-api/index.ts" %}{% endcodeBlock %}
     * @default null
     */
    template: string;
    /**
     * The ListView has an option to custom design the ListView header title with the help of headerTemplate property.
     * Refer the documentation [here]
     * (./listview/customizing-templates#header-template)
     *  to know more about this property with demo.
     *
     * {% codeBlock src="listview/headertemplate-api/index.ts" %}{% endcodeBlock %}
     * @default null
     */
    headerTemplate: string;
    /**
     * The ListView has an option to custom design the group header title with the help of groupTemplate property.
     * Refer the documentation [here]
     * (./listview/customizing-templates#group-template)
     *  to know more about this property with demo.
     *
     * {% codeBlock src="listview/grouptemplate-api/index.ts" %}{% endcodeBlock %}
     * @default null
     */
    groupTemplate: string;
    /**
     * We can trigger the `select` event when we select the list item in the component.
     * @event
     */
    select: EmitType<SelectEventArgs>;
    /**
     * We can trigger `actionBegin` event before every ListView action starts.
     * @event
     */
    actionBegin: EmitType<Object>;
    /**
     * We can trigger `actionComplete` event for every ListView action success event
     *  with the dataSource parameter.
     * @event
     */
    actionComplete: EmitType<Object>;
    /**
     * We can trigger `actionFailure` event for every ListView action failure event
     *  with the dataSource parameter.
     * @event
     */
    actionFailure: EmitType<Object>;
    /**
     * Constructor for creating the widget
     */
    constructor(options?: ListViewModel, element?: string | HTMLElement);
    onPropertyChanged(newProp: ListViewModel, oldProp: ListViewModel): void;
    private setHTMLAttribute;
    private setCSSClass;
    private setSize;
    private setEnable;
    private setEnableRTL;
    private enableElement;
    private header;
    private switchView;
    protected preRender(): void;
    private initialization;
    private renderCheckbox;
    private checkInternally;
    /**
     * It is used to check the checkbox of an item.
     * @param  {Fields | HTMLElement | Element} item - It accepts Fields or HTML list element as an argument.
     */
    checkItem(item: Fields | HTMLElement | Element): void;
    private toggleCheckBase;
    /**
     * It is used to uncheck the checkbox of an item.
     * @param  {Fields | HTMLElement | Element} item - It accepts Fields or HTML list element as an argument.
     */
    uncheckItem(item: Fields | HTMLElement | Element): void;
    /**
     * It is used to check all the items in ListView.
     */
    checkAllItems(): void;
    /**
     * It is used to un-check all the items in ListView.
     */
    uncheckAllItems(): void;
    private toggleAllCheckBase;
    private setCheckbox;
    /**
     * It is used to refresh the UI list item height
     */
    refreshItemHeight(): void;
    private clickHandler;
    private removeElement;
    private hoverHandler;
    private leaveHandler;
    private homeKeyHandler;
    private onArrowKeyDown;
    private arrowKeyHandler;
    private enterKeyHandler;
    private spaceKeyHandler;
    private keyActionHandler;
    private swipeActionHandler;
    private focusout;
    private wireEvents;
    private unWireEvents;
    private tabFocus;
    private removeFocus;
    private removeHover;
    private removeSelect;
    private isValidLI;
    private setCheckboxLI;
    private selectEventData;
    private setSelectedItemData;
    private setSelectLI;
    private setHoverLI;
    private getSubDS;
    private getItemData;
    private findItemFromDS;
    private getQuery;
    private setViewDataSource;
    private isInAnimation;
    private setLocalData;
    private reRender;
    private resetCurrentList;
    private createList;
    private renderSubList;
    private renderIntoDom;
    private renderList;
    private getElementUID;
    /**
     * It is used to Initialize the control rendering.
     */
    render(): void;
    /**
     * It is used to destroy the ListView component.
     */
    destroy(): void;
    /**
     * It helps to switch back from navigated sub list.
     */
    back(): void;
    /**
     * It is used to select the list item from the ListView.
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    selectItem(obj: Fields | HTMLElement | Element): void;
    private getLiFromObjOrElement;
    /**
     * It is used to select multiple list item from the ListView.
     * @param  {Fields[] | HTMLElement[] | Element[]} obj - We can pass array of elements or array of field Object with ID and Text fields.
     */
    selectMultipleItems(obj: Fields[] | HTMLElement[] | Element[]): void;
    private getParentId;
    /**
     * It is used to get the currently [here](./api-selectedItem)
     *  item details from the list items.
     */
    getSelectedItems(): SelectedItem | SelectedCollection | UISelectedItem | NestedListData;
    /**
     * It is used to find out an item details from the current list.
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    findItem(obj: Fields | HTMLElement | Element): SelectedItem;
    /**
     * A function that used to enable the disabled list items based on passed element.
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    enableItem(obj: Fields | HTMLElement | Element): void;
    /**
     * It is used to disable the list items based on passed element.
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    disableItem(obj: Fields | HTMLElement | Element): void;
    private setItemState;
    /**
     * It is used to show an list item from the ListView.
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    showItem(obj: Fields | HTMLElement | Element): void;
    /**
     * It is used to hide an item from the ListView.
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    hideItem(obj: Fields | HTMLElement | Element): void;
    private showHideItem;
    /**
     * It adds new item to current ListView.
     * To add a new item in the list view, we need to pass ‘data’ as array or object and ‘fields’ as object.
     * For example fields: { text: 'Name', tooltip: 'Name', id:'id'}
     * @param  {{[key:string]:Object}[]} data - Array JSON Data that need to add.
     * @param  {Fields} fields - Fields as an Object with ID and Text fields.
     */
    addItem(data: {
        [key: string]: Object;
    }[], fields?: Fields): void;
    private addListItem;
    /**
     * A function that removes the item from data source based on passed element like fields: { text: 'Name', tooltip: 'Name', id:'id'}
     * @param  {Fields | HTMLElement | Element} obj - We can pass element Object or Fields as Object with ID and Text fields.
     */
    removeItem(obj: Fields | HTMLElement | Element): void;
    private removeItemFromList;
    /**
     * A function that removes multiple item from list view based on given input.
     * @param  {Fields[] | HTMLElement[] | Element[]} obj - We can pass array of elements or array of field Object with ID and Text fields.
     */
    removeMultipleItems(obj: HTMLElement[] | Element[] | Fields[]): void;
    protected getModuleName(): string;
    requiredModules(): ModuleDeclaration[];
    /**
     * Get the properties to be maintained in the persisted state.
     */
    protected getPersistData(): string;
}
export interface ClassNames {
    root: string;
    hover: string;
    focused: string;
    selected: string;
    parentItem: string;
    listItem: string;
    hasChild: string;
    view: string;
    header: string;
    text: string;
    headerText: string;
    headerTemplateText: string;
    listItemText: string;
    listIcon: string;
    textContent: string;
    groupListItem: string;
    disable: string;
    content: string;
    backIcon: string;
    icon: string;
    checkboxWrapper: string;
    checkbox: string;
    checked: string;
    checkboxIcon: string;
    checklist: string;
    checkboxRight: string;
    checkboxLeft: string;
    listviewCheckbox: string;
    itemCheckList: string;
}
export interface SelectedItem {
    /**
     * It denotes the Selected Item text.
     */
    text: string;
    /**
     * It denotes the Selected Item list element.
     */
    item: HTMLElement | Element;
    /**
     * It denotes the Selected Item dataSource JSON object.
     */
    data: {
        [key: string]: Object;
    } | string[] | number[];
}
export interface SelectedCollection {
    /**
     * It denotes the Selected Item text data or collection.
     */
    text: string | string[];
    /**
     * It denotes the Selected Item list element or element collection.
     */
    item: HTMLElement | Element[] | NodeList;
    /**
     * It denotes the Selected Item dataSource JSON object or object collection.
     */
    data: {
        [key: string]: Object;
    } | {
        [key: string]: Object;
    }[] | string[] | number[];
}
export interface UISelectedItem {
    /**
     * It denotes the Selected Item text data or collection.
     */
    text: string | number | string[] | number[];
    /**
     * It denotes the Selected Item list element or element collection.
     */
    item?: HTMLElement | Element[] | NodeList;
    /**
     * It denotes the Selected Item dataSource JSON object or object collection.
     */
    data: {
        [key: string]: Object;
    } | {
        [key: string]: Object;
    }[] | string | number | string[] | number[];
    /**
     * It is used to denote the index of the selected element.
     */
    index?: number | number[];
    /**
     * It is used to check whether the element is checked or not.
     */
    isChecked?: boolean;
}
export interface DataAndParent {
    /**
     * It denotes the Selected Item dataSource JSON object or object collection.
     */
    data: {
        [key: string]: Object;
    } | {
        [key: string]: Object;
    }[] | string[];
    /**
     * It denotes the Selected Item's parent id;
     */
    parentId: string[];
}
export interface NestedListData {
    /**
     * It denotes the Selected Item text data or collection.
     */
    text: string | string[];
    /**
     * It denotes the Selected Item list element or element collection.
     */
    item: HTMLElement | Element[] | NodeList;
    /**
     * It denotes the Selected Item dataSource JSON object with it's parent ID.
     */
    data: DataAndParent[];
}
export interface SelectEventArgs extends BaseEventArgs, SelectedItem {
    /**
     * Specifies that event has triggered by user interaction.
     */
    isInteracted: boolean;
    /**
     * Specifies that event argument when event raised by other event.
     */
    event: MouseEvent | KeyboardEvent;
    /**
     * It is used to denote the index of the selected element.
     */
    index: number;
    /**
     * It is used to check whether the element is checked or not.
     */
    isChecked?: boolean;
}
export interface ItemCreatedArgs {
    curData: {
        [key: string]: Object;
    };
    dataSource: {
        [key: string]: Object;
    } | string[];
    fields: FieldsMapping;
    item: HTMLElement;
    options: ListBaseOptions;
    text: string;
}
